/**
 * @file bc7215_ac_lib.h
 * @brief BC7215 Air Conditioner Control Library Header File
 * @details This library provides functions to control air conditioners using the BC7215 chip.
 *          It supports various AC modes, fan speeds, and temperature control operations.
 * @version See bc7215_ac_get_ver() for current version
 * @date Created: 2025-05-20
 * @author Bitcode
 */

#ifndef BC7215_AC_LIB_H
#define BC7215_AC_LIB_H

#include <stdbool.h>
#include "bc7215_lib.h"

#ifdef __cplusplus
extern "C" {
#endif

/* ================================================================================================
 * AIR CONDITIONER MODE DEFINITIONS
 * ================================================================================================ */

/** @defgroup AC_Modes Air Conditioner Operating Modes
 * @brief Defines the available operating modes for the air conditioner
 * @{
 */
#define MODE_AUTO   0  /**< Automatic mode - AC adjusts cooling/heating automatically */
#define MODE_COOL   1  /**< Cooling mode - AC operates in cooling mode */
#define MODE_HOT    2  /**< Heating mode - AC operates in heating mode */
#define MODE_DRY    3  /**< Dry mode - AC removes humidity without significant temperature change */
#define MODE_FAN    4  /**< Fan only mode - AC operates as a fan without cooling/heating */
/** @} */

/* ================================================================================================
 * FAN SPEED DEFINITIONS
 * ================================================================================================ */

/** @defgroup Fan_Speeds Fan Speed Settings
 * @brief Defines the available fan speed settings
 * @{
 */
#define FAN_AUTO    0  /**< Automatic fan speed - AC adjusts fan speed automatically */
#define FAN_LOW     1  /**< Low fan speed */
#define FAN_MED     2  /**< Medium fan speed */
#define FAN_HIGH    3  /**< High fan speed */
/** @} */

/* ================================================================================================
 * CONTROL KEY DEFINITIONS
 * ================================================================================================ */

/** @defgroup Control_Keys Control Key Identifiers
 * @brief Defines the available control keys for AC operation
 * @{
 */
#define KEY_PLUS    0  /**< Temperature increase key */
#define KEY_MINUS   1  /**< Temperature decrease key */
#define KEY_MODE    2  /**< Mode selection key */
#define KEY_FAN     3  /**< Fan speed selection key */
/** @} */

/* ================================================================================================
 * PUBLIC FUNCTION DECLARATIONS
 * ================================================================================================ */

/**
 * @brief Get the version string of the BC7215 AC library
 * @return Pointer to a constant string containing the version information
 * @note The returned string is statically allocated and should not be modified
 */
const char* bc7215_ac_get_ver(void);

/**
 * @brief Initialize the BC7215 AC control library
 * @param status Status byte from the data packet
 * @param dataPktCool25C Reference data packet for cooling at 25°C
 * @return true if initialization successful, false otherwise
 * @note This function must be called before using any other library functions
 * @warning Ensure the dataPktCool25C parameter points to valid data
 */
bool bc7215_ac_init(uint8_t status, const bc7215DataVarPkt_t* dataPktCool25C);

/**
 * @brief Find the next available AC configuration
 * @return true if next configuration found, false if no more configurations available
 * @note This function is used to iterate through available AC configurations
 */
bool bc7215_ac_find_next(void);

/**
 * @brief Set AC parameters and generate corresponding data packet
 * @param temp Target temperature (0 to 14, refering 16°C to 30°C)
 * @param mode Operating mode (see MODE_* definitions)
 * @param fan Fan speed setting (see FAN_* definitions)
 * @param key Control key being pressed (see KEY_* definitions)
 * @return Pointer to generated data packet
 * @note The returned packet contains the IR signal data for the specified settings
 */
const bc7215DataVarPkt_t* bc7215_ac_set(int8_t temp, int8_t mode, int8_t fan, int8_t key);

/**
 * @brief Generate data packet to turn AC on
 * @return Pointer to data packet for turning AC on, NULL if no dedicated ON packet is needed
 * @note This function generates the appropriate IR signal to power on the AC
 */
const bc7215DataVarPkt_t* bc7215_ac_on(void);

/**
 * @brief Generate data packet to turn AC off
 * @return Pointer to data packet for turning AC off, NULL if operation failed
 * @note This function generates the appropriate IR signal to power off the AC
 */
const bc7215DataVarPkt_t* bc7215_ac_off(void);

/**
 * @brief Get the number of predefined AC configurations
 * @return Number of available predefined configurations
 * @note Use this to determine the valid range for bc7215_ac_predefined_* functions
 */
uint8_t bc7215_ac_predefined_cnt(void);

/**
 * @brief Get predefined AC configuration data by index
 * @param index Index of the predefined configuration (0 to bc7215_ac_predefined_cnt()-1)
 * @return Pointer to data packet for the specified configuration, NULL if index invalid
 * @warning Ensure index is within valid range to avoid undefined behavior
 */
const bc7215DataVarPkt_t* bc7215_ac_predefined_data(uint8_t index);

/**
 * @brief Get predefined AC configuration format packet by index
 * @param index Index of the predefined configuration (0 to bc7215_ac_predefined_cnt()-1)
 * @return Pointer to format packet for the specified configuration, NULL if index invalid
 * @warning Ensure index is within valid range to avoid undefined behavior
 */
const bc7215FormatPkt_t* bc7215_ac_predefined_fmt(uint8_t index);

/**
 * @brief Get predefined AC configuration name by index
 * @param index Index of the predefined configuration (0 to bc7215_ac_predefined_cnt()-1)
 * @return Pointer to string containing the configuration name, NULL if index invalid
 * @note The returned string is statically allocated and should not be modified
 * @warning Ensure index is within valid range to avoid undefined behavior
 */
const char* bc7215_ac_predefined_name(uint8_t index);


#ifdef __cplusplus
}
#endif

#endif /* BC7215_AC_LIB_H */
